package ru.dbotthepony.mc.otm.core.util

import it.unimi.dsi.fastutil.HashCommon
import net.minecraft.core.component.DataComponentMap
import net.minecraft.core.component.DataComponentPatch
import net.minecraft.core.component.PatchedDataComponentMap
import net.minecraft.core.registries.BuiltInRegistries
import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack
import ru.dbotthepony.mc.otm.core.getHolder

class ItemStackKey(val item: Item, val components: DataComponentPatch) {
	// make copy of original itemstack because there is no copy() method on DataComponentMap, which is returned by ItemStack#getComponents
	constructor(itemStack: ItemStack) : this(itemStack.item, itemStack.copy().componentsPatch)
	constructor(item: Item) : this(item, DataComponentPatch.EMPTY)

	private var hashComputed = false
	private var hash = 0

	override fun equals(other: Any?): Boolean {
		return this === other || other is ItemStackKey && other.item === item && other.components == components
	}

	override fun hashCode(): Int {
		if (!hashComputed) {
			hash = HashCommon.murmurHash3(item.hashCode().toLong().shl(32) or components.hashCode().toLong()).toInt()
			hashComputed = true
		}

		return hash
	}

	fun asItemStack(count: Int = 1): ItemStack {
		return ItemStack(BuiltInRegistries.ITEM.getHolder(item)!!, count, components)
	}

	override fun toString(): String {
		return "ItemStackKey[$item, $components]"
	}
}

fun ItemStack.asKey(): ItemStackKey {
	return ItemStackKey(this)
}

fun ItemStack.asKeyOrNull(): ItemStackKey? {
	if (isEmpty) return null
	return ItemStackKey(this)
}

fun Item.asKey(): ItemStackKey {
	return ItemStackKey(this)
}
